# auxiliary functions

function base(d::Int)
    id = I(d)
    b = Vector{CartesianIndex{d}}(undef,d) # base vectors
    for i = 1:d
        b[i] = CartesianIndex(id[i,:]...)
    end
    return b
end

const base2 = base(2)
const base3 = base(3)

# check node in domain
function in_domain(q;n=nmax)
    all(1 .<= Tuple(q) .<= n+1)
end

function in_obstacle(p,t;n=nmax)
    g(idx2pt(p),t/n) >= 0
end

# Neighbor calculation
# 3D version
function neighbors(p::CartesianIndex{3})
    return [p+s*e for e in base3 for s in [-1,1] if in_domain(p+s*e)]
end
# 2D version
function neighbors(p::CartesianIndex{2})
    return [p+s*e for e in base2 for s in [-1,1] if in_domain(p+s*e)]
end


"""
Point to index \\
transforms 0<=x<=1 into index 1:nmax+1, so that x=0 goes to 1 and x=1 goes to nmax +1
"""
function pt2idx(x;n=nmax)
    CartesianIndex(round.(Int,x*n.+1)...)
end

function pt2idx(x,dom;n=nmax)
    CartesianIndex(round.(Int,(x-dom[:,1])*n.+1)...)
end

function pt2int(x;n=nmax)
    round(Int,x*n+1)
end

"""
Index to real point coordinates in physical space 
"""
function idx2pt(p;n=nmax)
    dim = length(p)
    [p[i]-1 for i in 1:dim]/n
end

function dist(p,q)
    n = length(p); m=length(q)
    if m!=n
        error("points have different sizes")
    end
    sum((p[i]-q[i])^2 for i=1:n) |> sqrt
end

function dot(p,q)
    n = length(p); m=length(q)
    if m!=n
        error("points have different sizes")
    end
    sum(p[i]*q[i] for i = 1:n)
end

function norm(p)
    dot(p,p) |> sqrt
end
